#include "earliest_execution_table.h"

EarlistExecutionTable::~EarlistExecutionTable()
{
  delete[] table_;
}

EarlistExecutionTable::EarlistExecutionTable(
    risc::sg::SegmentGraph &segment_graph,
    risc::sg::EventConflictTable &event_conflict_table):
  segment_graph_(segment_graph),
  event_conflict_table_(event_conflict_table),
  time_advance_table(segment_graph_)
{
  size_ = event_conflict_table_.size_of_conflict_table_;
  table_ = new risc::sg::Time[size_];

  std::pair<risc::sg::VertexIterator, risc::sg::VertexIterator> vp;
  for(vp = boost::vertices(segment_graph_.graph_);
      vp.first != vp.second; ++vp.first) {

    table_index_segment_lookup_.push_back(vp.first);
  }

  determine_table();
}


void
EarlistExecutionTable::print_table()
{
  for(int i=0; i<size_; i++) {

    std::cout << segment_graph_.graph_[*table_index_segment_lookup_.at(i)].id_
              << ", "
              << table_[i]
              << std::endl;
  }
}

risc::sg::Time&
EarlistExecutionTable::get_min_time(risc::sg::VertexIterator vertex_iter)
{
  for(int i = 0; i < table_index_segment_lookup_.size(); i++) {

    if(table_index_segment_lookup_.at(i) == vertex_iter) {

      return table_[i];
    }
  }
}

void
EarlistExecutionTable::determine_table()
{
  for(int i=0; i<size_; i++) {

    std::pair<risc::sg::VertexIterator, risc::sg::VertexIterator> vp;

    risc::sg::VertexIterator temp;

    for(vp = boost::vertices(segment_graph_.graph_);
        vp.first != vp.second;
        ++vp.first) {

      if(segment_graph_.graph_[*(vp.first)].color_for_communication_graph_
          == risc::sg::Segment::Touched) {

        continue;
      }

      risc::sg::Graph::in_edge_iterator in_edge_begin, in_edge_end, in_edge_iter;
      boost::tie(in_edge_begin, in_edge_end)
        = boost::in_edges(*(vp.first), segment_graph_.graph_);

      if(in_edge_begin == in_edge_end) {
        temp = (vp.first);

        for(int i = 0; i < table_index_segment_lookup_.size(); i++) {
          if(table_index_segment_lookup_.at(i) == vp.first) {

            risc::sg::Time time;
            table_[i] = time;
          }
        }

        break;
      }

      bool all_predecessor_touched = true;
      for(in_edge_iter  = in_edge_begin;
          in_edge_iter != in_edge_end;
          in_edge_iter++) {

        if(boost::source(*in_edge_iter, segment_graph_.graph_) == *(vp.first)) {

          continue;
        }

        if(segment_graph_.graph_[
            boost::source(*in_edge_iter, segment_graph_.graph_)].
              color_for_communication_graph_
            == risc::sg::Segment::Untouched) {

          all_predecessor_touched = false;
        }
      }

      if(all_predecessor_touched) {

        temp = (vp.first);
        break;
      }

      temp = (vp.first);
    }

    segment_graph_.graph_[*temp].color_for_communication_graph_
      = risc::sg::Segment::Touched;

    risc::sg::Graph::in_edge_iterator in_edge_begin, in_edge_end, in_edge_iter;
    boost::tie(in_edge_begin, in_edge_end)
      = boost::in_edges(*temp, segment_graph_.graph_);


    risc::sg::Time waiting_time_for_segment
      = time_advance_table.get_waiting_time_for_segment(vp.first);

    risc::sg::Time min_time;
    bool first_iteration = true;
    for(in_edge_iter  = in_edge_begin;
        in_edge_iter != in_edge_end;
        in_edge_iter++) {

      if(first_iteration) {

        first_iteration = false;
        min_time = get_min_time(
                       std::find(vertices(segment_graph_.graph_).first,
                                vertices(segment_graph_.graph_).second,
                                boost::source(*in_edge_iter,
                                           segment_graph_.graph_)));

        min_time = min_time + waiting_time_for_segment;

      } else {

        risc::sg::Time temp = get_min_time(
                        std::find(vertices(segment_graph_.graph_).first,
                                 vertices(segment_graph_.graph_).second,
                                 boost::source(*in_edge_iter,
                                            segment_graph_.graph_)));

        if(min_time + waiting_time_for_segment < min_time) {
          min_time = temp + waiting_time_for_segment;
        } else {
          std::cout << "ouch" << std::endl;
        }
      }
    }

    get_min_time(vp.first) = min_time;
  }
}

void
EarlistExecutionTable::optimize_table()
{
  //for(int opt_step=0; opt_step<size_; opt_step++) {

    for(int time_entry=0; time_entry<size_; time_entry++) {

      //std::cout << time_entry << std::endl;
      for(int
          event_column = 0;
          event_column < event_conflict_table_.size_of_conflict_table_;
          event_column++) {

        if(event_conflict_table_.conflict_table_[event_column][time_entry]) {


  //        std::cout << "  Hello" << std::endl;

          if(table_[event_column] == table_[time_entry]) {


            std::cout << "A " << event_column << " -> " << time_entry << std::endl;
          table_[time_entry].delta_count_++;

          } else if(table_[event_column] < table_[time_entry]) {
            table_[time_entry] = table_[event_column];
            table_[time_entry].delta_count_++;
            std::cout << "B " << event_column << " -> " << time_entry << std::endl;
          }

        }
      }
      //std::cout << std::endl;
    }
  //}
}
