#include <iostream>

#include "risc.h"

int main(int argc, char* argv[])
{
  std::cout << std::endl
            << "start processing file: " __FILE__
            << " ..."
            << std::endl;

  risc::Design d(argc,argv);


  risc::sg::SegmentGraph::FunctionCallBoundarySet function_call_boundary(
      risc::tools::SystemCDefinitions::get_sc_wait_funcs());
  risc::sg::SegmentGraph sg(function_call_boundary);

  for(risc::ModuleVectorIter
      iter_module  = d.module_definitions_.begin();
      iter_module != d.module_definitions_.end();
      iter_module++) {

    std::cout << "Module: " <<  (*iter_module)->get_name() << std::endl;

    for(risc::ThreadVectorIter
        iter_thread = (*iter_module)->threads_.begin();
        iter_thread != (*iter_module)->threads_.end();
        iter_thread++) {

      std::cout << "  " << (*iter_thread)->get_name() << std::endl;
      sg.add_function(*iter_thread);
    }
    std::cout << std::endl;
  }

  sg.clean_graph();

  risc::sg::DataConflictTable dct(sg, d.path_instance_mapper_, true);
  if(d.perform_static_analysis()) {
    sg.print_graph("channel_variables.dot");
    sg.print_graph_read_write_access("channel_variables_rw_sg.dot");
    dct.print_conflict_table("channel_variables_data.html");
  }

  risc::sg::EventConflictTable ect(sg, d.path_instance_mapper_, true);
  if(d.perform_static_analysis()) {
    ect.print_conflict_table("channel_variables_event.html");
    ect.print_lookup_table("channel_variables_lookup.html");
  }

  d.path_instance_mapper_->print_mapping();

  std::cout << "end processing file: " __FILE__
            << " ..."
            << std::endl
            << std::endl;

  return 0;
}

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
