#include <iostream>

#include "risc.h"

int main(int argc, char* argv[])
{
  std::cout << std::endl
            << "start processing file: " __FILE__
            << " ..."
            << std::endl;

  risc::Design d(argc,argv);

  d.print_design();

  risc::sg::SegmentGraph::FunctionCallBoundarySet function_call_boundary(
      risc::tools::SystemCDefinitions::get_sc_wait_funcs());
  risc::sg::SegmentGraph sg(function_call_boundary);

  for(risc::ModuleVectorIter
      iter_module  = d.module_definitions_.begin();
      iter_module != d.module_definitions_.end();
      iter_module++) {

    std::cout << "Module: " <<  (*iter_module)->get_name() << std::endl;

    for(risc::ThreadVectorIter
        iter_thread = (*iter_module)->threads_.begin();
        iter_thread != (*iter_module)->threads_.end();
        iter_thread++) {

      std::cout << "  " << (*iter_thread)->get_name() << std::endl;

      sg.add_function(*iter_thread);
    }

    std::cout << std::endl;
  }

  sg.clean_graph();

  risc::sg::DataConflictTable dct(sg, d.path_instance_mapper_, true);
  risc::sg::EventConflictTable ect(sg, d.path_instance_mapper_, true);

  if(d.perform_static_analysis()) {
    sg.print_graph("data_conflict_table_function_annotation_static.dot");
    sg.print_graph_read_write_access("data_conflict_table_read_write_function_annotation_static.dot");
    dct.print_conflict_table("data_conflict_table_function_annotation_static.html");
    ect.print_conflict_table("event_conflict_table_function_annotation_static.html");
    ect.print_lookup_table("event_lookup_table_function_annotation_static.html");


  } else {

    sg.print_graph("data_conflict_table_function_annotation_dynamic.dot");
    sg.print_graph_read_write_access("data_conflict_table_read_write_function_annotation_dynamic.dot");
    dct.print_conflict_table("data_conflict_table_function_annotation_dynamic.html");
    ect.print_conflict_table("event_conflict_table_function_annotation_dynamic.html");
    ect.print_lookup_table("event_lookup_table_function_annotation_dynamic.html");
  }

  std::cout << "end processing file: " __FILE__
            << " ..."
            << std::endl
            << std::endl;

  return backend(d.project_);
  //return 0;
}

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
